﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using MvcDemo.Areas.HttpHeaders.Models;
using MvcDemo.Areas.HttpHeaders.ViewModels;
using NWebsec.Mvc.HttpHeaders;

namespace MvcDemo.Areas.HttpHeaders.Controllers
{
    public class CacheTestingController : Controller
    {
        //
        // GET: /HttpHeaders/CacheControlTest/

        public ActionResult Index()
        {
            return View();
        }

        [XRobotsTag(NoIndex = true, NoFollow = true)]
        public ActionResult Test(CacheSettings settings)
        {

            Mapper.CreateMap<CacheSettings, CacheSettingsData>();
            var viewData = Mapper.Map<CacheSettings, CacheSettingsData>(settings);
            SetCounterValue(viewData);
            SetResponseCachePolicy(settings);
            return View(viewData);
        }

        public ActionResult Post(CacheSettings settings)
        {

            Mapper.CreateMap<CacheSettings, CacheSettingsData>();
            var viewData = Mapper.Map<CacheSettings, CacheSettingsData>(settings);
            SetCounterValue(viewData);
            SetResponseCachePolicy(settings);
            return View(viewData);
        }

        public ActionResult Redirect(CacheSettings settings)
        {
            Mapper.CreateMap<CacheSettings, CacheSettingsData>();
            var viewData = Mapper.Map<CacheSettings, CacheSettingsData>(settings);
            SetCounterValue(viewData);
            SetResponseCachePolicy(settings);
            return RedirectToAction("Get", new { viewData.Cacheability, viewData.NoStore, viewData.MustRevalidate, viewData.Expires, viewData.MaxAge, viewData.Pragma });
        }

        public ActionResult Get(CacheSettings settings)
        {

            Mapper.CreateMap<CacheSettings, CacheSettingsData>();
            var viewData = Mapper.Map<CacheSettings, CacheSettingsData>(settings);
            SetCounterValue(viewData);
            SetResponseCachePolicy(settings);
            return View(viewData);
        }
        private void SetResponseCachePolicy(CacheSettings settings)
        {
            switch (settings.Cacheability)
            {
                case Cacheability.Public:
                    Response.Cache.SetCacheability(HttpCacheability.Public);
                    break;
                case Cacheability.Private:
                    Response.Cache.SetCacheability(HttpCacheability.Private);
                    break;
                case Cacheability.NoCache:
                    Response.Cache.SetCacheability(HttpCacheability.NoCache);
                    break;
            }

            if (settings.NoStore)
                Response.Cache.SetNoStore();

            if (settings.MustRevalidate)
                Response.Cache.SetRevalidation(HttpCacheRevalidation.AllCaches);

            if (settings.Expires)
            {
                if (settings.Cacheability != Cacheability.NoCache)
                {
                    Response.AppendHeader("Expires", "-1");
                }
            }
            else
            {
                //Remove expires header added automagically with no-cache
                if (settings.Cacheability == Cacheability.NoCache)
                    Response.Cache.SetAllowResponseInBrowserHistory(true);
            }

            if (settings.MaxAge)
                Response.Cache.SetMaxAge(TimeSpan.Zero);

            if (settings.Pragma)
                Response.AppendHeader("Pragma", "no-cache");
        }

        private void SetCounterValue(CacheSettingsData cacheSettings)
        {
            if (Request.Cookies["CacheCounter"] == null || cacheSettings.ResetCounter)
            {
                SetCounterValueCookie(1);
                cacheSettings.Counter = 1;
                return;
            }

            var cookie = Request.Cookies["CacheCounter"];
            var count = 0;
            var isNumber = Int32.TryParse(cookie.Value, out count);

            if (!isNumber)
            {

                SetCounterValueCookie(1);
                cacheSettings.Counter = 1;
                cacheSettings.UserMessage = "So, the cookie did not contain a valid number. Are you being funny?";
                return;
            }

            if (count == Int32.MaxValue)
            {
                SetCounterValueCookie(1);
                cacheSettings.Counter = 1;
                cacheSettings.UserMessage = "The counter has reached Int32.MaxValue eh? You must have been clicking all day. The counter has been reset, keep clicking! :)";
                return;
            }
            var newCount = count + 1;
            SetCounterValueCookie(newCount);
            cacheSettings.Counter = newCount;
        }

        private void SetCounterValueCookie(int value)
        {
            Response.SetCookie(new HttpCookie("CacheCounter", value.ToString(CultureInfo.InvariantCulture)) { Secure = false, HttpOnly = true });
        }
    }
}
