﻿using System;
using System.Web;
using System.Web.Mvc;
using NWebsec.Mvc.HttpHeaders;

namespace MvcDemo.Areas.SessionSecurity.Controllers
{
    [SetNoCacheHttpHeaders]
    public class CookieTestController : Controller
    {
        private const string CookieName = "TestCookie";
        //
        // GET: /Cookies/

        [XRobotsTag(NoFollow = true)]
        public ActionResult Index()
        {
            return View();
        }

        [XRobotsTag(NoIndex = true, NoFollow = true)]
        public ActionResult SetHostCookie(bool clear = false)
        {
            Response.AppendCookie(GetCookie(false, clear));
            return RedirectToAction("Index");
        }

        [XRobotsTag(NoIndex = true, NoFollow = true)]
        public ActionResult SetDomainCookie(bool clear = false)
        {
            Response.AppendCookie(GetCookie(true, clear));
            return RedirectToAction("Index");
        }

        [XRobotsTag(NoIndex = true, NoFollow = true)]
        public ActionResult SetHostDomainCookie(bool clear = false)
        {
            //These will come out in reversed order
            Response.AppendCookie(GetCookie(true, clear));
            Response.AppendCookie(GetCookie(false, clear));
            return RedirectToAction("Index");
        }

        [XRobotsTag(NoIndex = true, NoFollow = true)]
        public ActionResult SetDomainHostCookie()
        {
            //These will come out in reversed order
            Response.AppendCookie(GetCookie(false, false));
            Response.AppendCookie(GetCookie(true, false));
            return RedirectToAction("Index");
        }

        private HttpCookie GetCookie(bool domain, bool clear)
        {
            var value = domain ? "domaincookie" : "hostcookie";
            value += "_" + DateTime.UtcNow.ToString("HH:mm:ss");
            var c = new HttpCookie(CookieName, value);
            c.Domain = domain ? ".nwebsec.com" : String.Empty;

            if (clear)
            {
                c.Expires = DateTime.UtcNow.AddYears(-10);
            }

            return c;
        }
    }
}
