﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;

namespace MvcDemo.Helpers
{
    public class BreadCrumbHelper
    {
        private HttpRequestBase _request;
        private HtmlHelper _helper;
        public BreadCrumbHelper(HttpRequestBase request, HtmlHelper helper)
        {
            _request = request;
            _helper = helper;
        }

        public HtmlString CreateBreadCrumbLinks()
        {
            if (_request.Url.AbsolutePath.Equals("/"))
                return new HtmlString(String.Empty);

            var pathComponents = _request.Url.AbsolutePath.Trim(new[] { '/' }).Split(new[] { '/' });
            
            var breadCrumbs = new List<String>();
            var linkFormat = "<a href=\"{0}\">{1}</a>";
            breadCrumbs.Add(String.Format(linkFormat, "/", "Home"));

            var currentPath = new StringBuilder();
            for (int i = 0; i < pathComponents.Length - 1; i++)
            {
                var component = pathComponents[i];
                if (String.IsNullOrEmpty(component) || component.Equals("Home"))
                    continue;
                currentPath.Append("/");
                currentPath.Append(component);
                var link = String.Format(linkFormat, _helper.AttributeEncode(currentPath.ToString()), _helper.Encode(component));
                breadCrumbs.Add(link);
            }
            breadCrumbs.Add(_helper.Encode(pathComponents[pathComponents.Length -1]));
            return breadCrumbs.Count < 2 ? new HtmlString(String.Empty) : new HtmlString(String.Join(" > ", breadCrumbs));
        }
    }
}